import {
  select,
  csv,
  scaleLinear,
  scaleBand,
  scaleOrdinal,
  max,
  axisLeft,
  axisBottom,
} from 'd3';

const width = 960;
const height = 450;
const margin = {
  top: 50,
  right: 50,
  bottom: 50,
  left: 75,
};

const innerWidth = width - margin.left - margin.right;
const innerHeight = height - margin.top - margin.bottom;

const dateFormatter = new Intl.DateTimeFormat('en-US', {
  month: 'short',
  year: '2-digit',
});

var BrowserText = (function () {
  var canvas = document.createElement('canvas'),
    context = canvas.getContext('2d');

  /**
   * Measures the rendered width of arbitrary text given the font size and font face
   **/
  function getWidth(text, selector) {
    const element = document.querySelector(selector);
    const style = getComputedStyle(element);
    context.font = style.fontSize + ' ' + style.fontFamily;
    return context.measureText(text).width;
  }

  return {
    getWidth: getWidth,
  };
})();

const g = select('body')
  .append('svg')
  .attr('width', width)
  .attr('height', height)
  .append('g')
  .attr(
    'transform',
    `translate(${margin.left},${margin.top})`
  );

// const g = svg
//   .append('g')
//   .attr(
//     'transform',
//     `translate(${margin.left},${margin.top})`
//   );

const render = (data) => {
  const xValue = (d) => d.readDate;
  const yValue = (d) => d.totalKwh;
  const colorValue = (d) => d.accountNumber;

  const dataByDate = d3.group(data, (d) => d.readDate);

  const transformedData = [];
  for (const [date, originalRows] of dataByDate) {
    const row = { date };
    console.log(date);
    console.log(row);
    for (const d of originalRows) {
      row[d.accountNumber] = d.totalKwh;
    }
    transformedData.push(row);
  }
console.log("transformedData");
console.log(transformedData);
  
  const domain = [...new Set(data.map((d) => xValue(d)))];

  const accounts = [
    ...new Set(data.map((d) => d.accountNumber)),
  ];
  const stackedData = d3.stack().keys(accounts)(
    transformedData
  );

  // var divToolTip = d3
  //   .select('body')
  //   .append('div')
  //   .attr('class', 'tooltip')
  //   .style('opacity', 0)
  //   .html('tooltip');

  const xScale = scaleBand()
    .domain(domain)
    .range([0, innerWidth])
    .padding(0.3);
  g.append('g')
    .attr('transform', `translate(0,${innerHeight})`)
    .call(axisBottom(xScale));
  g.append('text')
    .attr('text-anchor', 'middle')
    .attr('x', innerWidth / 2)
    .attr('y', innerHeight + margin.bottom)
    .attr('fill', 'darkblue')
    .text('Month');

  // TODO use max here
  const yScale = scaleLinear()
    .domain([2000, 0])
    .range([0, innerHeight]);
  g.append('g').call(axisLeft(yScale));
  g.append('text')
    .attr('text-anchor', 'middle')
    .attr('x', -innerHeight / 2)
    .attr('y', -margin.left / 2 - 10)
    .attr('transform', 'rotate(-90)')
    .attr('fill', 'darkblue')
    .text('Electricity Usage Kwh');

  const color = scaleOrdinal()
    .domain(accounts)
    .range(d3.schemeCategory10);
  // const barColor = (d) => {
  //   const result = color(colorValue(d));
  //   return result;
  // };

  g.append('g')
    .selectAll('g')
    // Enter in the stack data = loop key per key = group per group
    .data(stackedData)
    .join('g')
    .attr('fill', (d) => color(d.key))
    .selectAll('rect')
    // enter a second time = loop subgroup per subgroup to add all rectangles
    .data((d) => d)
    .join('rect')
    .attr('x', (d) => xScale(d.data.date))
    .attr('y', (d) => yScale(d[1]))
    .attr('height', (d) => yScale(d[0]) - yScale(d[1]))
    .attr('width', xScale.bandwidth())
    .on('mouseover', function (d) {
      d3.select(this).attr('opacity', 0.75);
      //    	const barDate = d.target.__data__.data.date;
      //			const barTotal = dataByDate.get(barDate).reduce((acc, d) => acc + d.totalKwh, 0);
      //      divToolTip
      //        .transition()
      //        .duration(250)
      //        .style('opacity', 1);
      //console.log(xScale(barDate) + "," + yScale(barTotal));
      //    divToolTip
      //        .html(barTotal)
      //        .style('left', (xScale(barDate) + margin.left + margin.right) + 'px')
      //        .style('bottom', -1 * yScale(barTotal) + innerHeight + 'px');
    })
    .on('mouseout', function (d) {
      d3.select(this).transition().duration(250);
      d3.select(this).attr('opacity', 1);
      //      divToolTip
      //        .transition()
      //        .duration(500)
      //        .style('opacity', 0);
    })
    .append('title')
    .text((d, i) => (d[1] - d[0]) + ' kWH');

  g.append('text')
    .attr('text-anchor', 'middle')
    .attr('x', innerWidth / 2)
    .attr('y', -margin.top / 2)
    .style('font-size', '24px')
    .style('text-decoration', 'underline')
    .text('Personal Electric Consumption');

  // Add one dot in the legend for each name.
  const longestActNumber = accounts.reduce(
    (previous, current) =>
      previous.length > current.length ? previous : current
  );
  const textWidth = BrowserText.getWidth(
    longestActNumber,
    'text'
  );
  const element = document.querySelector('text');
  const style = getComputedStyle(element);
  const fontHeight = Number(
    style.fontSize.replace(/px$/, '')
  );

  const leftLegendPos =
    innerWidth - textWidth - fontHeight - 20;
  g.append('text')
    .attr('text-anchor', 'right')
    .attr('x', leftLegendPos)
    .attr('y', fontHeight - 20)
    .style('font-size', '16px')
    .style('text-decoration', 'underline')
    .text('Accounts');

  g.selectAll('legenddots')
    .data(accounts)
    .enter()
    .append('rect')
    .attr('x', leftLegendPos)
    .attr('y', function (d, i) {
      return 0 + i * (fontHeight + 10);
    })
    .attr('width', style.fontSize)
    .attr('height', style.fontSize)
    .style('fill', function (d) {
      return color(d);
    });

  g.selectAll('legendlabels')
    .data(accounts)
    .enter()
    .append('text')
    .attr('x', innerWidth - textWidth - 10)
    .attr('y', function (d, i) {
      return i * (fontHeight + 10) + fontHeight;
    })
    .style('fill', function (d) {
      return color(d);
    })
    .text(function (d) {
      return d;
    })
    .attr('text-anchor', 'left')
    .style('alignment-baseline', 'middle');
};

const csvUrl = [
  'https://gist.githubusercontent.com/',
  'pcordone/', // User
  '93039d5d909fbf42838c5167219d6ed4/', // Id of the Gist
  'raw/c60b15781b63168a096404fa4be4ac2a252ee0c1/', // commit
  'electricAccountsData.csv', // File name
].join('');

csv(csvUrl).then((data) => {
  data.forEach((d) => {
    d.readDate = dateFormatter.format(new Date(d.readDate));
    d.readDays = +d.readDays;
    d.totalKwh = +d.totalKwh;
    d.utilityCharges = +d.utilityCharges;
    d.supplierCharges = +d.supplierCharges;
    d.totalCharges = +d.totalCharges;
    d.avgDailyUsage = +d.avgDailyUsage;
    d.readFromDate = new Date(d.readFromDate);
    d.readToDate = new Date(d.readToDate);
    d.relativeMonthBillDate = new Date(
      d.relativeMonthBillDate
    );
    d.timestamp = new Date(d.timestamp);
    d.netMeterCredits = +d.netMeterCredits;
  });
  render(data);
});